top_50_metros = [
    {
        "Common Name": "New York",
        "Official MSA Name": "New York-Newark-Jersey City",
        "parcl_id": 2900187,
    },
    {
        "Common Name": "Los Angeles",
        "Official MSA Name": "Los Angeles-Long Beach-Anaheim",
        "parcl_id": 2900078,
    },
    {
        "Common Name": "Chicago",
        "Official MSA Name": "Chicago-Naperville-Elgin",
        "parcl_id": 2899845,
    },
    {
        "Common Name": "Dallas",
        "Official MSA Name": "Dallas-Fort Worth-Arlington",
        "parcl_id": 2899734,
    },
    {
        "Common Name": "Houston",
        "Official MSA Name": "Houston-Pasadena-The Woodlands",
        "parcl_id": 2899967,
    },
    {
        "Common Name": "Washington",
        "Official MSA Name": "Washington-Arlington-Alexandria",
        "parcl_id": 2900475,
    },
    {
        "Common Name": "Philadelphia",
        "Official MSA Name": "Philadelphia-Camden-Wilmington",
        "parcl_id": 2900244,
    },
    {
        "Common Name": "Atlanta",
        "Official MSA Name": "Atlanta-Sandy Springs-Roswell",
        "parcl_id": 2887280,
    },
    {
        "Common Name": "Miami",
        "Official MSA Name": "Miami-Fort Lauderdale-West Palm Beach",
        "parcl_id": 2900128,
    },
    {
        "Common Name": "Phoenix",
        "Official MSA Name": "Phoenix-Mesa-Chandler",
        "parcl_id": 2900245,
    },
    {
        "Common Name": "Boston",
        "Official MSA Name": "Boston-Cambridge-Newton",
        "parcl_id": 2899625,
    },
    {
        "Common Name": "Riverside",
        "Official MSA Name": "Riverside-San Bernardino-Ontario",
        "parcl_id": 2900295,
    },
    {
        "Common Name": "San Francisco",
        "Official MSA Name": "San Francisco-Oakland-Fremont",
        "parcl_id": 2900336,
    },
    {
        "Common Name": "Detroit",
        "Official MSA Name": "Detroit-Warren-Dearborn",
        "parcl_id": 2899753,
    },
    {
        "Common Name": "Seattle",
        "Official MSA Name": "Seattle-Tacoma-Bellevue",
        "parcl_id": 2900353,
    },
    {
        "Common Name": "Minneapolis",
        "Official MSA Name": "Minneapolis-St. Paul-Bloomington",
        "parcl_id": 2900137,
    },
    {
        "Common Name": "Tampa",
        "Official MSA Name": "Tampa-St. Petersburg-Clearwater",
        "parcl_id": 2900417,
    },
    {
        "Common Name": "San Diego",
        "Official MSA Name": "San Diego-Chula Vista-Carlsbad",
        "parcl_id": 2900332,
    },
    {
        "Common Name": "Denver",
        "Official MSA Name": "Denver-Aurora-Centennial",
        "parcl_id": 2899750,
    },
    {
        "Common Name": "Baltimore",
        "Official MSA Name": "Baltimore-Columbia-Towson",
        "parcl_id": 2887292,
    },
    {"Common Name": "St. Louis", "Official MSA Name": "St. Louis", "parcl_id": 2900321},
    {
        "Common Name": "Orlando",
        "Official MSA Name": "Orlando-Kissimmee-Sanford",
        "parcl_id": 2900213,
    },
    {
        "Common Name": "Charlotte",
        "Official MSA Name": "Charlotte-Concord-Gastonia",
        "parcl_id": 2899841,
    },
    {
        "Common Name": "San Antonio",
        "Official MSA Name": "San Antonio-New Braunfels",
        "parcl_id": 2900331,
    },
    {
        "Common Name": "Portland",
        "Official MSA Name": "Portland-Vancouver-Hillsboro",
        "parcl_id": 2900266,
    },
    {
        "Common Name": "Pittsburgh",
        "Official MSA Name": "Pittsburgh",
        "parcl_id": 2900251,
    },
    {
        "Common Name": "Austin",
        "Official MSA Name": "Austin-Round Rock-San Marcos",
        "parcl_id": 2887289,
    },
    {
        "Common Name": "Sacramento",
        "Official MSA Name": "Sacramento-Roseville-Folsom",
        "parcl_id": 2900315,
    },
    {
        "Common Name": "Las Vegas",
        "Official MSA Name": "Las Vegas-Henderson-North Las Vegas",
        "parcl_id": 2900049,
    },
    {
        "Common Name": "Cincinnati",
        "Official MSA Name": "Cincinnati",
        "parcl_id": 2899647,
    },
    {
        "Common Name": "Kansas City",
        "Official MSA Name": "Kansas City",
        "parcl_id": 2900012,
    },
    {"Common Name": "Columbus", "Official MSA Name": "Columbus", "parcl_id": 2899671},
    {"Common Name": "Cleveland", "Official MSA Name": "Cleveland", "parcl_id": 2899654},
    {
        "Common Name": "Indianapolis",
        "Official MSA Name": "Indianapolis-Carmel-Greenwood",
        "parcl_id": 2899979,
    },
    {
        "Common Name": "Nashville",
        "Official MSA Name": "Nashville-Davidson-Murfreesboro-Franklin",
        "parcl_id": 2900174,
    },
    {
        "Common Name": "San Jose",
        "Official MSA Name": "San Jose-Sunnyvale-Santa Clara",
        "parcl_id": 2900338,
    },
    {
        "Common Name": "Virginia Beach",
        "Official MSA Name": "Virginia Beach-Chesapeake-Norfolk",
        "parcl_id": 2900462,
    },
    {
        "Common Name": "Jacksonville",
        "Official MSA Name": "Jacksonville",
        "parcl_id": 2899989,
    },
    {
        "Common Name": "Providence",
        "Official MSA Name": "Providence-Warwick",
        "parcl_id": 2900275,
    },
    {
        "Common Name": "Milwaukee",
        "Official MSA Name": "Milwaukee-Waukesha",
        "parcl_id": 2900134,
    },
    {
        "Common Name": "Raleigh",
        "Official MSA Name": "Raleigh-Cary",
        "parcl_id": 2900282,
    },
    {
        "Common Name": "Oklahoma City",
        "Official MSA Name": "Oklahoma City",
        "parcl_id": 2900205,
    },
    {
        "Common Name": "Louisville",
        "Official MSA Name": "Louisville\/Jefferson County",
        "parcl_id": 2900079,
    },
    {"Common Name": "Memphis", "Official MSA Name": "Memphis", "parcl_id": 2900122},
    {"Common Name": "Richmond", "Official MSA Name": "Richmond", "parcl_id": 2900292},
    {
        "Common Name": "Salt Lake City",
        "Official MSA Name": "Salt Lake City-Murray",
        "parcl_id": 2900329,
    },
    {
        "Common Name": "Birmingham",
        "Official MSA Name": "Birmingham",
        "parcl_id": 2899611,
    },
    {"Common Name": "Fresno", "Official MSA Name": "Fresno", "parcl_id": 2899715},
    {
        "Common Name": "Buffalo",
        "Official MSA Name": "Buffalo-Cheektowaga",
        "parcl_id": 2899645,
    },
    {
        "Common Name": "Hartford",
        "Official MSA Name": "Hartford-West Hartford-East Hartford",
        "parcl_id": 2899944,
    },
    {
        "Common Name": "Grand Rapids",
        "Official MSA Name": "Grand Rapids-Wyoming-Kentwood",
        "parcl_id": 2899916,
    },
    {"Common Name": "Tucson", "Official MSA Name": "Tucson", "parcl_id": 2900436},
]
